/**
 * @fileoverview ChromaDB JavaScript client library entry point.
 * Provides access to all public APIs for interacting with ChromaDB.
 */

// Apply Deno compatibility patch
import "./deno";

export { Collection } from "./collection";
export { withChroma } from "./next";
export * from "./types";
export * from "./admin-client";
export * from "./chroma-client";
export * from "./embedding-function";
export * from "./cloud-client";
export * from "./errors";
export * from "./collection-configuration";
export * from "./execution";
export * from "./schema";
